﻿namespace SerialPWMController
{



    #region Using Statements:



    using System;
    using System.IO.Ports;
    using System.Windows.Forms;



    #endregion




    public partial class Form1 : Form
    {



        #region Fields:



        /// <summary>
        /// The Microcontroller Clock Frequency:
        /// </summary>
        int fclock = 16000000;



        /// <summary>
        /// The Clock Frequency Prescaler:
        /// </summary>
        int Prescaler = 1024;



        /// <summary>
        /// Create a new SerialPort Reference.
        /// <para>
        ///     See: https://docs.microsoft.com/en-us/dotnet/api/system.io.ports.serialport?view=dotnet-plat-ext-3.1
        /// </para>
        /// </summary>
        SerialPort serialPort = new SerialPort();



        #endregion



        #region Properties:



        /// <summary>
        /// Serial Port Name:
        /// </summary>
        public string PortName
        {
            get { return serialPort.PortName; }
            internal set { serialPort.PortName = value; }
        }



        /// <summary>
        /// Serial Port BaudRate:
        /// </summary>
        public int BaudRate
        {
            get { return serialPort.BaudRate; }
            internal set { serialPort.BaudRate = value; }
        }



        /// <summary>
        /// Serial Port Parity:
        /// </summary>
        public Parity Parity
        {
            get { return serialPort.Parity; }
            internal set { serialPort.Parity = value; }
        }



        /// <summary>
        /// Serial Port DataBits:
        /// </summary>
        public int DataBits
        {
            get { return serialPort.DataBits; }
            internal set { serialPort.DataBits = value; }
        }



        /// <summary>
        /// Serial Port StopBits:
        /// </summary>
        public StopBits StopBits
        {
            get { return serialPort.StopBits; }
            internal set { serialPort.StopBits = value; }
        }



        /// <summary>
        /// Serial Port Handshake:
        /// </summary>
        public Handshake Handshake
        {
            get { return serialPort.Handshake; }
            internal set { serialPort.Handshake = value; }
        }



        #endregion



        /// <summary>
        /// Form Constructor.
        /// </summary>
        public Form1()
        {

            // Form Init:
            InitializeComponent();

            // Add Port Names:
            foreach (string PortName in SerialPort.GetPortNames())
                if (!PortNameComboBox.Items.Contains(PortName))
                    PortNameComboBox.Items.Add(PortName);

            // Standard Baud Rates:
            int[] BaudRates = new int[] { 110, 300, 600, 1200, 2400, 4800, 9600, 14400, 19200, 38400, 57600, 115200, 128000, 256000 };

            // Add Port Names:
            foreach (int BaudRate in BaudRates)
                BaudRateComboBox.Items.Add(BaudRate);

            // Set Selected Index:
            BaudRateComboBox.SelectedItem = 115200;
        }



        /// <summary>
        /// Form Load Event.
        /// </summary>
        private void Form1_Load(object sender, EventArgs e)
        {

            // Set the read/write timeouts
            serialPort.ReadTimeout = 500;
            serialPort.WriteTimeout = 500;
        }



        /// <summary>
        /// Calculate the Frequency:
        /// </summary>
        /// <param name="frequency">The desired Frequency: 3.73Hz</param>
        /// <returns>int representing the resolution on the Register (16 bits).</returns>
        int CalculateFrequency(double frequency)
        {

            // Calculate OCR1A from specified Frequency:
            return fclock / (int)(frequency * Prescaler) - 1;
        }



        /// <summary>
        /// Calculate the Duty Cycle:
        /// </summary>
        /// <param name="period">Period, or 1 / F</param>
        /// <param name="duty">The desired Duty Cycle: 50.0%</param>
        /// <returns>int representing the resolution on the Register (16 bits).</returns>
        int CalculateDutyCycle(int period, double duty)
        {

            // Calculate OCR1B from OCR1A and specified Duty Cycle:
            return (short)(period * (duty / 100L));
        }



        /// <summary>
        /// Calculate Register Resolution:
        /// </summary>
        /// <param name="ocrx">The Register Value, E.G: CalculateFrequency(1024.57)</param>
        /// <returns>double representing the Resolution at the current Register Value.</returns>
        double CalculateResolution(int ocrx)
        {

            return (Math.Log((double)ocrx + 1.0D) / Math.Log(2.0D));
        }



        /// <summary>
        /// Send Button Click Event.
        /// </summary>
        private void SendButton_Click(object sender, EventArgs e)
        {

            // Parse the Values:
            double.TryParse(FrequencyTextBox.Text, out double Frequency);
            double.TryParse(DutyCycleTextBox.Text, out double DutyCycle);

            // Calculate Resolutions:
            int FrequencyResolution = CalculateFrequency(Frequency);
            int DutyCycleResolution = CalculateDutyCycle(FrequencyResolution, DutyCycle);

            // Set Resolution:
            ResolutionLabel.Text = CalculateResolution(FrequencyResolution).ToString();

            // Send the Data:
            serialPort.WriteLine(String.Format("<{0}|{1}>", FrequencyResolution, DutyCycleResolution));
        }



        /// <summary>
        /// Serial Data Received Event.
        /// </summary>
        private void SerialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            // Handle Data here...
            string data = ((SerialPort)sender).ReadExisting();
            Invoke(new Action(() => {
                richTextBox1.AppendText(data + Environment.NewLine);
            }));
        }



        /// <summary>
        /// Connect Button Click Event.
        /// </summary>
        private void ConnectButton_Click(object sender, EventArgs e)
        {

            if (ConnectButton.Text == "Connect")
            {

                // Configure Serial Properties:
                string Port = PortNameComboBox.SelectedItem.ToString();
                int Baud = Convert.ToInt32(BaudRateComboBox.SelectedItem.ToString());
                serialPort = new SerialPort(Port);
                BaudRate = Baud;
                Parity = Parity.None;
                DataBits = 8;
                StopBits = StopBits.One;
                Handshake = Handshake.None;

                // 
                serialPort.RtsEnable = true;

                // Attach Data Reveived Event:
                serialPort.DataReceived += SerialPort_DataReceived;

                // Open the Serial Port:
                serialPort.Open();

                // Change Button Text:
                ConnectButton.Text = "Disconnect";

                // Return:
                return;
            }

            if (ConnectButton.Text == "Disconnect")
            {

                // Close the Serial Port:
                serialPort.Close();
                serialPort.Dispose();

                // Change Button Text:
                ConnectButton.Text = "Connect";

                // Return:
                return;
            }
        }
    }
}